from tkinter import Tk, filedialog

from asr_app.settings import settings


def browse_dir() -> str:
    root = Tk()
    root.attributes("-topmost", True)
    root.focus_force()
    root.withdraw()

    dir_path = filedialog.askdirectory(
        initialdir=settings.browse_dir_initial_dir,
    )

    root.destroy()

    return dir_path


def browse_audio_files() -> tuple[str]:
    root = Tk()
    root.attributes("-topmost", True)
    root.focus_force()
    root.withdraw()

    file_paths = filedialog.askopenfilenames(
        initialdir=settings.browse_files_initial_dir,
        filetypes=(("Audio files", settings.audio_files_ext),)
    )

    root.destroy()

    return file_paths


def browse_audio_files_str() -> str:
    file_paths = browse_audio_files()
    return "\n".join(file_paths)



title = "Transkrypcja audio"
menu_header: str = "# Menu"
files_label: str = "Pliki do transkrypcji:"
browse_files_btn: str = "Wybierz pliki"
dir_label: str = "Folder do zapisu:"
browse_dir_btn: str = "Wybierz folder"
model_dropdown_label: str = "Model"
transcribe_btn: str = "Wykonaj transkrypcję"
results_header: str = "# Wyniki"
results_table_header: list[str] = ["Plik audio", "Plik tekstowy"]
loading_model: str = "Trwa ładowanie modelu:"
loaded_model: str = "Załadowano model:"
file_not_exist: str = "Plik nie istnieje."
processing_file: str = "Trwa przetwarzanie pliku:"
processing_file_ended: str = "Zakończono przetwarzanie pliku:"
processing_file_error: str = "Wystąpił błąd podczas transkrypcji."
saving_file_error: str = "Wystąpił błąd podczas zapisu do pliku."



import sys
import threading
from abc import ABC
from typing import Union

import tqdm

_hooked = False
_thread_local = threading.local()


class _CustomProgressBar(tqdm.tqdm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._current = self.n
        self.update(n=0)

    def update(self, n=1):
        super().update(n)
        self._current += n

        listeners = _get_thread_local_listeners()

        for listener in listeners:
            listener.on_progress(self._current, self.total)


def _get_thread_local_listeners():
    if not hasattr(_thread_local, 'listeners'):
        _thread_local.listeners = []
    return _thread_local.listeners


def init_progress_hook():
    global _hooked

    if _hooked:
        return

    # Inject into tqdm.tqdm of Whisper, so we can see progress
    transcribe_module = sys.modules['whisper.transcribe']
    transcribe_module.tqdm.tqdm = _CustomProgressBar
    _hooked = True


class ProgressListener(ABC):
    def on_progress(self, current: Union[int, float], total: Union[int, float]):
        self.total = total

    def on_finished(self):
        pass


class ProgressListenerHandle:
    def __init__(self, listener: ProgressListener):
        self.listener = listener

    def __enter__(self):
        register_thread_local_progress_listener(self.listener)

    def __exit__(self, exc_type, exc_val, exc_tb):
        unregister_thread_local_progress_listener(self.listener)

        if exc_type is None:
            self.listener.on_finished()


def register_thread_local_progress_listener(progress_listener: ProgressListener):
    # This is a workaround for the fact that the progress bar is not exposed in the API
    init_progress_hook()

    listeners = _get_thread_local_listeners()
    listeners.append(progress_listener)


def unregister_thread_local_progress_listener(progress_listener: ProgressListener):
    listeners = _get_thread_local_listeners()

    if progress_listener in listeners:
        listeners.remove(progress_listener)


def progress_listener_handle(progress_listener: ProgressListener):
    return ProgressListenerHandle(progress_listener)



from pathlib import Path
from typing import Union

import gradio as gr
import pandas as pd
import whisper
from whisper import Whisper

import asr_app.texts as t
from asr_app.io import browse_audio_files_str, browse_dir
from asr_app.progress import ProgressListener, progress_listener_handle
from asr_app.settings import settings

model: Union[Whisper | None] = None


def load_model(model_name: str, model_name_loaded: str, progress=gr.Progress()) -> tuple[str, str]:
    global model

    if model_name != model_name_loaded:
        progress((0, None), t.loading_model, unit="Ładowanie modelu")
        model = whisper.load_model(model_name)
        progress(None)

    return model_name, f"#### {t.loaded_model} {model_name}"


def on_transcribe(files_paths: str, save_dir: str, progress=gr.Progress()):
    files_paths = [Path(path) for path in files_paths.split('\n')]

    class FileProgressListener(ProgressListener):
        def __init__(self, file_path: Path):
            self.progress_description = f"{t.processing_file} {file_path}"
            self.finished_message = f"{t.processing_file_ended} {file_path}"

        def on_progress(self, current: Union[int, float], total: Union[int, float]):
            progress(progress=(current, total), desc=self.progress_description)

        def on_finished(self):
            gr.Info(message=self.finished_message)

    results = []
    for path in files_paths:

        if not path.exists():
            results.append((str(path), t.file_not_exist))
            continue

        listener = FileProgressListener(file_path=path)
        with progress_listener_handle(listener):
            try:
                result = model.transcribe(str(path), verbose=False, language="pl")
            except Exception as e:
                results.append((str(path), t.processing_file_error))
                continue
        try:
            save_path = get_save_path(save_dir, path)
            save_path.write_text(result["text"], encoding='utf-8')
        except Exception as e:
            results.append((str(path), t.saving_file_error))
            continue

        results.append((str(path), str(save_path)))

    df = pd.DataFrame(results, columns=t.results_table_header)
    return df


def get_save_path(save_dir: str, file_path: Path) -> Path:
    if save_dir and Path(save_dir).is_dir():
        save_dir = Path(save_dir)
    else:
        save_dir = file_path.parent

    return (save_dir / file_path.name).with_suffix(".txt")


def change_describe_button(text: str) -> gr.Button:
    if text.strip() == '':
        return gr.Button(value=t.transcribe_btn, variant="primary", min_width=1, interactive=False)
    else:
        return gr.Button(value=t.transcribe_btn, variant="primary", min_width=1, interactive=True)


with gr.Blocks(title=t.title) as demo:
    model_name_loaded_state = gr.State('')

    with gr.Row():
        with gr.Column(scale=2):
            menu_header = gr.Markdown(t.menu_header)

            with gr.Accordion(label=t.files_label, open=True):
                input_paths = gr.Markdown()
                browse_button = gr.Button(
                    value=t.browse_files_btn,
                    variant="secondary",
                )
                browse_button.click(
                    browse_audio_files_str,
                    outputs=input_paths,
                    show_progress="hidden",
                )

            with gr.Accordion(label=t.dir_label, open=True):
                output_dir = gr.Markdown()
                browse_button_dir = gr.Button(
                    value=t.browse_dir_btn,
                    variant="secondary",
                )
                browse_button_dir.click(
                    browse_dir,
                    outputs=output_dir,
                    show_progress="hidden",
                )

            model_dropdown = gr.Dropdown(
                label=t.model_dropdown_label,
                choices=settings.whisper_models_names,
                value=settings.whisper_default_model,
            )
            model_info = gr.Markdown()
            transcribe_button = gr.Button(
                value=t.transcribe_btn,
                variant="primary",
                min_width=1,
                interactive=False
            )

            input_paths.change(change_describe_button,
                               inputs=input_paths,
                               outputs=transcribe_button)

        with gr.Column(scale=5):
            header = gr.Markdown(t.results_header)
            output_values = gr.DataFrame(
                headers=t.results_table_header,
                col_count=(2, "fixed"),
                type="pandas",
                interactive=False,
            )
            file_content = gr.Markdown()


    def on_select(value: pd.DataFrame, evt: gr.SelectData):
        path = Path(value.iat[evt.index[0], 1])

        if path.is_file():
            text = path.read_text(encoding='utf-8')
        else:
            text = t.file_not_exist

        return text

    output_values.select(on_select, output_values, file_content)

    transcribe_button.click(
        load_model,
        inputs=[model_dropdown, model_name_loaded_state],
        outputs=[model_name_loaded_state, model_info],
    ).then(
        on_transcribe,
        inputs=[input_paths, output_dir],
        outputs=output_values,
    )


def main():
    demo.queue().launch()


if __name__ == '__main__':
    main()



from typing import Tuple

from pydantic import Field
from pydantic_settings import BaseSettings, SettingsConfigDict


class Settings(BaseSettings):
    browse_files_initial_dir: str = Field(".")
    browse_dir_initial_dir: str = Field(".")

    audio_files_ext: Tuple[str, ...] = Field(
        default=('*.flac', '*.m4a', '*.mp3', '*.mp4', '*.mpeg', '*.mpga', '*.oga', '*.ogg',
                 '*.wav', '*.webm'),
    )

    whisper_models_names: list[str] = ['base', 'small', 'medium', 'large']
    whisper_default_model: str = 'large'

    model_config = SettingsConfigDict(env_file='.env', env_file_encoding='utf-8', extra='ignore')


settings = Settings()
